# EventdClient
#
# Communicates with an em-websocket web socket instance, wrapping it in an evented layer. Implements the
# Eventd Websocket Communication Protocol described at:
#
# https://github.com/hydrais/shot/wiki/Eventd-Websocket-Communication-Protocol
#
# Shot Framework - Copyright (c) Jesse Aaron Dunlap <me@jessedunlap.me>
# Licensed under the MIT License. For full licensing information, please
# see LICENSE.md. http://github.com/JesseDunlap/shot/

require 'json'

require_relative './eventd_object'
require_relative './eventd_element'

class EventdClient < EventdObject

	# em-websocket socket object
	attr_accessor :socket

	# Initializer
	#
	# == Attributes
	# * +socket+
	# * +server+

	def initialize(socket, server = nil)
		super()
		@socket = socket
    @server = server
		setup_socket unless @socket == nil
	end

	# Handle required events from the socket

	def setup_socket
		@socket.onopen do |handshake|
			self.emit('connect', handshake, true)
		end

		@socket.onclose do
			self.emit('disconnect', nil, true)
		end

		@socket.onmessage do |message|
			begin
				emission = JSON.parse message

				if emission['broadcast']
					self.broadcast emission['channel'], emission['data']
				end

				self.emit emission['channel'], emission['data']
				self.emit('message', message, true)
			rescue JSON::ParserError
				self.emit('message', message, true)
			end
		end
	end

	# Send an emission to the client side library
	#
	# == Attributes
	# * +channel+
	# * +data+
	# * +local+

	def emit(channel, data = nil, local = false)
		super(channel, data)

		if not local
			emission = { 'channel' => channel, 'priority' => 'normal', 'data' => data }
			@socket.send emission.to_json
		end
  end

  # Send a broadcast to other clients
  #
  # == Attributes
  # * +channel+
  # * +data+

  def broadcast(channel, data = nil)
    if @server.broadcast_channel_allowed? channel
      @server.clients.each do |client| client.emit channel, data end
    else
      self.emit 'error', "Broadcasts are not allowed on channel: #{channel}"
    end
  end

  # Instantiate a new EventdElement
  #
  # == Attributes
  # * +selector+
  
  def element(selector)
  	EventdElement.new(selector, self)
  end
end