# EventdObject
#
# Implements an event-based architecture centered around the concept of *channels* and *listeners*. Each channel can
# contain multiple listeners. An "emission" can be sent on a channel, which will be then sent to all corresponding
# listeners on that channel. Emissions can have optional data sent with them to the listeners.
#
# Shot Framework - Copyright (c) Jesse Aaron Dunlap <me@jessedunlap.me>
# Licensed under the MIT License. For full licensing information, please
# see LICENSE.md. http://github.com/JesseDunlap/shot/


class EventdObject
	attr_accessor :events

	# Initialize the EventdObject and set up the events hash.
	# Note:: It is important that you always call +super()+ so that the +events+ hash can be initialized

	def initialize
		@events = {}
	end

	# Add an event listener to a specified channel
	#
	# == Attributes
	# * +channel+ - Channel to subscribe to
	# * +callback+ - Callback block, which is triggered when an emission occurs on the channel

	def on(channel, &callback)
		@events[channel] = (@events[channel] or [])
		@events[channel].push callback
	end

	# Remove all event listeners from a specified channel
	#
	# == Attributes
	# * +channel+ - Channel to unsubscribe from

	def off(channel)
		@events[channel] = []
	end

	# Send an emission on a channel
	#
	# == Attributes
	# * +channel+ - Channel to emit on
	# * +data+ - Optional. Data to send along with the emission.

	def emit(channel, data = nil)
		emit 'emission', { :channel => channel, :data => data } unless channel == 'emission'

		@events[channel] = (@events[channel] or [])

		@events[channel].each do |callback|
			if data == nil then callback.call else callback.call data end
		end
	end
end