# EventdServer
#
# Wraps the em-websocket library in an event-based system. All incoming clients are also converted
# to EventdClients, which implement the Eventd Websocket Communication Protocol described at:
#
# https://github.com/hydrais/shot/wiki/Eventd-Websocket-Communication-Protocol
#
# Shot Framework - Copyright (c) Jesse Aaron Dunlap <me@jessedunlap.me>
# Licensed under the MIT License. For full licensing information, please
# see LICENSE.md. http://github.com/JesseDunlap/shot/


require 'em-websocket'
require_relative './eventd_object'
require_relative './eventd_client'

class EventdServer < EventdObject
	# Configuration options for em-websocket and the EventdServer
	attr_accessor :options

	# Callback block which is called when the server starts for threading purposed
	attr_accessor :run_callback

  # List of active clients on the server
  attr_accessor :clients

  # List of allowed broadcast channels
  attr_accessor :allowed_broadcast_channels

  # Broadcast restriction
  attr_accessor :broadcasts_restricted

	# Initialize the EventdServer with configuration options
	#
	# == Attributes
	# * +options+ - Configuration hash which will be used as the initial EventdServer configuration

	def initialize(options = { :host => '127.0.0.1', :port => 8080 })
		super()
		@options = options

    @clients = []
    @broadcasts_restricted = false
    @allowed_broadcast_channels = []
	end

	# Add to the EventdServer configuration
	#
	# == Attributes
	# * +options+ - Configuration hash to merge with the existing configuration

	def configure(options)
		@options.merge! options
	end

	# Start the server

	def start
		EM.run do
			@run_callback.call

			EM::WebSocket.run @options do |socket|
				client = EventdClient.new socket, self
				self.emit 'connection', client

        self.clients.push client
        handle_disconnection_for client
			end
		end
	end

	# Special event listener which is triggered when the server runs

	def run(&callback)
		@run_callback = callback
  end

  # Allows you to extend the functionality of a server, through a plugin

  def plugin(&callback)
    callback.call self
  end

  # Allow any broadcast

  def allow_broadcasts
    @broadcasts_restricted = false
  end

  # Restrict broadcasts

  def restrict_broadcasts
    @broadcasts_restricted = true
  end

  # Add an allowed broadcast channel

  def add_allowed_broadcast(channel)
    @allowed_broadcast_channels.push channel
  end

  # Remove an allowed broadcast channel

  def remove_allowed_broadcast(channel)
    @allowed_broadcast_channels.delete_if do |c| c == channel end
  end

  # Check if broadcasts are allowed

  def broadcasts_allowed?
    !@broadcasts_restricted
  end

  # Check if broadcasts are restricted

  def broadcasts_restricted?
    @broadcasts_restricted
  end

  # Check if a specific broadcast channel is allowed

  def broadcast_channel_allowed?(channel)
    !@broadcasts_restricted or @allowed_broadcast_channels.include? channel
  end

  # Check if a specific broadcast channel is restricted

  def broadcast_channel_restricted?(channel)
    @allowed_broadcast_channels.find_index channel == nil and @broadcasts_restricted
  end

  private

  def handle_disconnection_for(client)
    client.on 'disconnect' do
      @clients.delete_if do |c| c == client end
      self.emit 'disconnection', client
    end
  end
end